%%%-------------------------------------------------------------------
%%% File    : sybase_worker.erl
%%% Author  : MP Scholtz (michael@erlfinsys.net)
%%% Description : Server for Erlang Sybase Driver
%%%
%%% Created : 2005/08/16
%%%-------------------------------------------------------------------
-module(sybase_worker).

-behaviour(gen_server).
%%--------------------------------------------------------------------
%% Include files
%%--------------------------------------------------------------------

%%--------------------------------------------------------------------
%% External exports
-export([start_link/1]).

%% gen_server callbacks
-export([init/1, handle_call/3, handle_cast/2, handle_info/2, terminate/2, code_change/3]).

-record(state, {port, rows, lastrow}).

-define(SERVER, ?MODULE).


%%====================================================================
%% External functions
%%====================================================================
%%--------------------------------------------------------------------
%% Function: start_link/0
%% Description: Starts the server
%%--------------------------------------------------------------------
start_link([Settings, Num]) ->
    gen_server:start_link(?MODULE, [Settings, Num], []).

%%====================================================================
%% Server functions
%%====================================================================

%%--------------------------------------------------------------------
%% Function: init/1
%% Description: Initiates the server
%% Returns: {ok, State}          |
%%          {ok, State, Timeout} |
%%          ignore               |
%%          {stop, Reason}
%%--------------------------------------------------------------------
init([Settings, Num]) ->
    DriverName = "sybasedrv" ++ integer_to_list(Num),
    case erl_ddll:load_driver(".", DriverName) of
	ok -> ok;
	{error, already_loaded} -> ok;
	_ -> exit({error, could_not_load_driver})
    end,
    Port = open_port({spawn, DriverName}, []),
    Port ! {self(), {command, [1, Settings]}}, %%Send connect to sybase_driver
    {ok, #state{port = Port, rows = []}}.

%%--------------------------------------------------------------------
%% Function: handle_call/3
%% Description: Handling call messages
%% Returns: {reply, Reply, State}          |
%%          {reply, Reply, State, Timeout} |
%%          {noreply, State}               |
%%          {noreply, State, Timeout}      |
%%          {stop, Reason, Reply, State}   | (terminate/2 is called)
%%          {stop, Reason, State}            (terminate/2 is called)
%%--------------------------------------------------------------------
handle_call(Request, From, State) ->
io:format("~n Unhandled Request - ~p~n", [Request]),
    {reply, ok, State}.			
%%--------------------------------------------------------------------
%% Function: handle_cast/2
%% Description: Handling cast messages
%% Returns: {noreply, State}          |
%%          {noreply, State, Timeout} |
%%          {stop, Reason, State}            (terminate/2 is called)
%%--------------------------------------------------------------------
handle_cast({rpc, Cmd}, State) ->
    {Proc, ArgList} = Cmd,
    X = create_string(ArgList, atom_to_list(Proc)),
    State#state.port!{self(), {command, [2, X]}},
    {noreply, State};

    
handle_cast(Msg, State) ->
    {noreply, State}.
%%--------------------------------------------------------------------
%% Function: handle_infox/2
%% Description: Handling all non call/cast messages
%% Returns: {noreply, State}          |
%%          {noreply, State, Timeout} |
%%          {stop, Reason, State}            (terminate/2 is called)
%%--------------------------------------------------------------------
handle_info({_, {data, [0]}}, State) ->
    case State#state.lastrow of
	undefined ->
	    NewState = State;
	_ ->
	    Rows = State#state.rows -- [State#state.lastrow],
	    io:format("Rows : ~p ~nStatus : ~p", [Rows, State#state.lastrow]),
	    NewState = State#state{lastrow = undefined,
				   rows = []}
    end,
    {noreply, NewState};
handle_info({_, {data, Data}}, State) ->
    Cols = string:tokens(Data, [173]),
    [H|T] = lists:reverse(Cols),
    Row = lists:reverse(T),
    NewState = State#state{lastrow = Row, rows = State#state.rows ++ [Row]},
    {noreply, NewState};

handle_info(Info, State) ->
    {noreply, State}.

%%--------------------------------------------------------------------
%% Function: terminate/2
%% Description: Shutdown the server
%% Returns: any (ignored by gen_server)
%%--------------------------------------------------------------------
terminate(Reason, State) ->
    ok.

%%--------------------------------------------------------------------
%% Func: code_change/3
%% Purpose: Convert process state when code is changed
%% Returns: {ok, NewState}
%%--------------------------------------------------------------------
code_change(OldVsn, State, Extra) ->
    {ok, State}.

%%--------------------------------------------------------------------
%%% Internal functions
%%--------------------------------------------------------------------
create_string([], String) ->
    [String|[0]];
create_string([{Arg, Type}|T], String) when is_list(Arg) ->
    create_string(T, String ++ [0] ++ Arg ++ [0] ++ [Type]);
create_string([{Arg, Type}|T], String) when is_integer(Arg) ->
    create_string(T, String ++ [0] ++ integer_to_list(Arg) ++ [0] ++ [Type]).

